package com.rjconsultores.ventaboletos.web.gui.controladores.relatorios;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zul.Bandbox;
import org.zkoss.zul.Checkbox;
import org.zkoss.zul.Combobox;
import org.zkoss.zul.Comboitem;
import org.zkoss.zul.Datebox;
import org.zkoss.zul.Paging;
import org.zkoss.zul.Radiogroup;
import org.zkoss.zul.Row;

import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.Estado;
import com.rjconsultores.ventaboletos.entidad.FormaPago;
import com.rjconsultores.ventaboletos.entidad.Moneda;
import com.rjconsultores.ventaboletos.entidad.PuntoVenta;
import com.rjconsultores.ventaboletos.enums.IndStatusBoleto;
import com.rjconsultores.ventaboletos.relatorios.impl.RelatorioBilhetesVendidos;
import com.rjconsultores.ventaboletos.relatorios.impl.RelatorioBilhetesVendidosFiltroFormaPagamento;
import com.rjconsultores.ventaboletos.relatorios.impl.RelatorioBilhetesVendidosNovoLayout;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio;
import com.rjconsultores.ventaboletos.service.EmpresaService;
import com.rjconsultores.ventaboletos.service.EstadoService;
import com.rjconsultores.ventaboletos.service.FormaPagoService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;
import com.rjconsultores.ventaboletos.web.utilerias.MyComboboxEstandar;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.MyTextbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderRelatorioVendasBilheteiro;

@Controller("relatorioBilhetesVendidosController")
@Scope("prototype")
public class RelatorioBilhetesVendidosController extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;

	@Autowired
	private DataSource dataSourceRead;

	@Autowired
	private EmpresaService empresaService;

	private Datebox datInicial;
	private Datebox datFinal;
	private MyComboboxEstandar cmbEmpresa;
	private MyComboboxEstandar cmbEstado;
	private Combobox cmbMoeda;
	private List<Empresa> lsEmpresa;
	private List<Estado> lsEstado;
	@Autowired
	private transient PagedListWrapper<PuntoVenta> plwPuntoVenta;
	private MyTextbox txtNombrePuntoVenta;
	private Bandbox bbPesquisaPuntoVenta;
	private MyListbox puntoVentaList;
	private MyListbox puntoVentaSelList;
	private Paging pagingPuntoVenta;
	private Checkbox chkVendido;
	private Checkbox chkCancelado;
	private Checkbox chkIdaVolta;
	private Checkbox chkEmitido;
	private Checkbox chkFormaPagamento;
	private Radiogroup rgLayout;
	@Autowired
	private EstadoService estadoService;
	private List<FormaPago> lsFormaPago;
	private MyComboboxEstandar cmbFormaPago;
	@Autowired
	private FormaPagoService formaPagoService;
	private Row rowFiltroFormaPagamento;

	@Override
	public void doAfterCompose(Component comp) throws Exception {
		lsEmpresa = empresaService.obtenerTodos();
		lsEstado = estadoService.obtenerTodos();
		super.doAfterCompose(comp);

		chkVendido.setChecked(true);
		chkCancelado.setChecked(true);
		puntoVentaList.setItemRenderer(new RenderRelatorioVendasBilheteiro());
		lsFormaPago = formaPagoService.obtenerTodos();
		cmbFormaPago.setAutocomplete(true);

	}

	public List<Empresa> getLsEmpresa() {
		return lsEmpresa;
	}

	public void setLsEmpresa(List<Empresa> lsEmpresa) {
		this.lsEmpresa = lsEmpresa;
	}

	public List<Estado> getLsEstado() {
		return lsEstado;
	}

	public void setLsEstado(List<Estado> lsEstado) {
		this.lsEstado = lsEstado;
	}

	private void executarPesquisaAgencia() {
		HibernateSearchObject<PuntoVenta> puntoVentaBusqueda = new HibernateSearchObject<>(PuntoVenta.class, pagingPuntoVenta.getPageSize());

		puntoVentaBusqueda.addFilterILike("nombpuntoventa", "%" + txtNombrePuntoVenta.getValue() + "%");
		puntoVentaBusqueda.addFilterEqual("activo", Boolean.TRUE);

		puntoVentaBusqueda.addSortAsc("nombpuntoventa");

		puntoVentaBusqueda.addFilterEqual("activo", Boolean.TRUE);

		plwPuntoVenta.init(puntoVentaBusqueda, puntoVentaList, pagingPuntoVenta);

		if (puntoVentaList.getData().length == 0) {
			try {
				Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
						Labels.getLabel("relatorioVendasBilheteiroController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
				ex.printStackTrace();
			}
		}
	}

	public void onClick$btnPesquisa(Event ev) {
		executarPesquisaAgencia();
	}

	public void onDoubleClick$puntoVentaSelList(Event ev) {
		PuntoVenta puntoVenta = (PuntoVenta) puntoVentaSelList.getSelected();
		puntoVentaSelList.removeItem(puntoVenta);
	}

	public void onDoubleClick$puntoVentaList(Event ev) {
		PuntoVenta puntoVenta = (PuntoVenta) puntoVentaList.getSelected();
		puntoVentaSelList.addItemNovo(puntoVenta);
	}

	public void onClick$btnLimpar(Event ev) {
		puntoVentaList.setData(new ArrayList<PuntoVenta>());

		bbPesquisaPuntoVenta.setText("");
	}

	public void onClick$btnExecutarRelatorio(Event ev) throws Exception {
		executarRelatorio(false);
	}

	public void onClick$btnExecutarRelatorioNovoLayout(Event ev) throws Exception {
		executarRelatorio(true);
	}

	/**
	 * @throws Exception
	 * 
	 */
	@SuppressWarnings({ "rawtypes", "unchecked" })
	private void executarRelatorio(boolean novoLayout) throws Exception {
		Relatorio relatorio;
		Map<String, Object> parametros = new HashMap<String, Object>();
		StringBuilder filtro = new StringBuilder();

		filtro.append("Agência: ");
		String puntoVentaIds = "";
		String puntoVentas = "";
		List<PuntoVenta> lsPuntoVentaSelecionados = new ArrayList(Arrays.asList(puntoVentaSelList.getData()));
		if (lsPuntoVentaSelecionados.isEmpty()) {
			puntoVentas = "Todas";
		} else {
			for (int i = 0; i < lsPuntoVentaSelecionados.size(); i++) {
				PuntoVenta puntoVenta = lsPuntoVentaSelecionados.get(i);
				puntoVentas = puntoVentas + puntoVenta.getNombpuntoventa() + ",";

				puntoVentaIds = puntoVentaIds + puntoVenta.getPuntoventaId() + ",";
			}

			// removendo ultima virgula
			puntoVentaIds = puntoVentaIds.substring(0, puntoVentaIds.length() - 1);
			puntoVentas = puntoVentas.substring(0, puntoVentas.length() - 1);
			parametros.put("NUMPUNTOVENTA", puntoVentaIds);
		}		
		
		filtro.append(puntoVentas).append(";");
		
		filtro.append(" Moeda: ");
		Comboitem itemMoeda = cmbMoeda.getSelectedItem();
		if (itemMoeda != null ) {
			Moneda moeda = (Moneda) itemMoeda.getValue();
			parametros.put("MOEDA_ID", moeda.getMonedaId());
			parametros.put("MOEDA_SIMBOLO", moeda.getSimbolo());
			filtro.append(moeda.getDescmoneda() + ";");
		}

		parametros.put("DATA_INICIAL", (java.util.Date) this.datInicial.getValue());
		parametros.put("DATA_FINAL", (java.util.Date) this.datFinal.getValue());
		parametros.put("NOME_RELATORIO", Labels.getLabel("relatorioBilhetesVendidosController.window.title"));
		parametros.put("USUARIO", UsuarioLogado.getUsuarioLogado().getNombusuario());

		String status = "";
		if (chkCancelado.isChecked()) {
			status += "'" + IndStatusBoleto.C + "',";
		}
		if (chkVendido.isChecked()) {
			status += "'" + IndStatusBoleto.V + "',";
		}
		if (status.length() > 0) {
			status = status.substring(0, status.length() - 1);
		} else {
			if (!chkIdaVolta.isChecked()) //caso não tenha selecionado nada traz vendidos 
				status += "'" + IndStatusBoleto.V + "'";
		}
			
			
		parametros.put("STATUS", status);

		filtro.append(" Empresa: ");

		Comboitem itemEmpresa = cmbEmpresa.getSelectedItem();
		if (itemEmpresa != null) {
			Empresa empresa = (Empresa) itemEmpresa.getValue();
			parametros.put("EMPRESA_ID", empresa.getEmpresaId());
			parametros.put("EMPRESA", empresa.getNombempresa());
			filtro.append(empresa.getNombempresa() + ";");
		} else {
			filtro.append(" Todas;");
		}
		
		parametros.put("TIPO_DATA", rgLayout.getSelectedItem().getValue());
		
		Comboitem itemEstado = cmbEstado.getSelectedItem();
		if (itemEstado != null) {
			Estado estado = (Estado) itemEstado.getValue();
			parametros.put("ESTADO_ID", estado.getEstadoId());
			parametros.put("ESTADO", estado.getNombestado());
		} else {
			parametros.put("ESTADO", "TODOS");
		}
		if (chkIdaVolta.isChecked()){
			parametros.put("IDAVOLTA", "1");
			filtro.append(" Ida / Volta;");
		}else {
			parametros.put("IDAVOLTA", "0");
		}
		
		FormaPago formaPago = null;

		if (cmbFormaPago.getSelectedItem() != null) {
			 formaPago = ((FormaPago) cmbFormaPago.getSelectedItem().getValue());
			parametros.put("FORMAPAGO_ID", formaPago.getFormapagoId());
			filtro.append(formaPago.getDescpago()).append("; ");
		}
		if(chkFormaPagamento.isChecked()) {
			relatorio = new RelatorioBilhetesVendidosFiltroFormaPagamento(parametros, dataSourceRead.getConnection());
		}else if (novoLayout) {
			relatorio = new RelatorioBilhetesVendidosNovoLayout(parametros, dataSourceRead.getConnection());
			parametros.put("TITULO", Labels.getLabel("relatorioBilhetesVendidosController.window.title"));
		} else {
			relatorio = new RelatorioBilhetesVendidos(parametros, dataSourceRead.getConnection());
		}
		parametros.put("FILTROS", filtro.toString());

		Map args = new HashMap();
		args.put("relatorio", relatorio);

		openWindow("/component/reportView.zul", Labels.getLabel("relatorioBilhetesVendidosController.window.title"), args, MODAL);
	}
	
	public void onClick$chkFormaPagamento(Event event) {
		rowFiltroFormaPagamento.setVisible(chkFormaPagamento.isChecked());
	}

	public Combobox getCmbMoeda() {
		return cmbMoeda;
	}

	public void setCmbMoeda(Combobox cmbMoeda) {
		this.cmbMoeda = cmbMoeda;
	}

	public Checkbox getChkEmitido() {
		return chkEmitido;
	}

	public void setChkEmitido(Checkbox chkEmitido) {
		this.chkEmitido = chkEmitido;
	}
	
	public List<FormaPago> getLsFormaPago() {
		return lsFormaPago;
	}
	
	public void setLsFormaPago(List<FormaPago> lsFormaPago) {
		this.lsFormaPago = lsFormaPago;
	}
}
